import asyncio
import string

from pyrogram.types import Message
from PIL import Image
from ddddocr import DdddOcr

from ...utils import async_partial
from ...data import get_datas
from .base import Monitor


class MistyMonitor(Monitor):
    ocr = "digit5-teko@v1"
    lock = asyncio.Lock()

    name = "Misty"
    chat_name = "FreeEmbyGroup"
    chat_user = "MistyNoiceBot"
    chat_keyword = r"空余名额数: (?!0$)"
    bot_username = "EmbyMistyBot"
    notify_create_name = True

    async def init(self, initial=True):
        async with self.lock:
            if isinstance(self.ocr, str):
                data = []
                files = (f"{self.ocr}.onnx", f"{self.ocr}.json")
                async for p in get_datas(self.basedir, files, proxy=self.proxy, caller=self.name):
                    if p is None:
                        self.log.info(f"初始化错误: 无法下载所需文件.")
                        return False
                    else:
                        data.append(p)
                self.__class__.ocr = DdddOcr(
                    show_ad=False, import_onnx_path=str(data[0]), charsets_path=str(data[1])
                )

        self.captcha = None
        self.log.info(f"正在初始化机器人状态.")
        wr = async_partial(self.client.wait_reply, self.bot_username)
        for _ in range(20 if initial else 3):
            try:
                msg: Message = await wr("/cancel")
                if msg.caption and "选择您要使用的功能" in msg.caption or msg.text:
                    msg = await wr("🌏切换服务器")
                    if "选择您要使用的服务器" in msg.text or msg.caption:
                        msg = await wr("✨Misty")
                        if "选择您要使用的功能" in msg.caption or msg.text:
                            msg = await wr("⚡️账号功能")
                if msg.text and "请选择功能" in msg.text or msg.caption:
                    msg = await wr("⚡️注册账号")
                    if "请输入验证码" in msg.caption or msg.text:
                        data = await self.client.download_media(msg, in_memory=True)
                        image = Image.open(data)
                        self.captcha = (
                            self.ocr.classification(image)
                            .translate(str.maketrans("", "", string.punctuation))
                            .replace(" ", "")
                        )
                        self.log.debug(f"接收到验证码: {self.captcha}")
            except (asyncio.TimeoutError, TypeError):
                continue
            else:
                if self.captcha and len(self.captcha) == 5:
                    self.log.info(f"机器人状态初始化完成, 当接收到邀请码时将输入验证码 {self.captcha} 以抢注, 请勿操作 @EmbyMistyBot.")
                    return True
                else:
                    self.log.info(f"机器人状态初始化失败, 正在重试.")
        else:
            self.log.bind(notify=True).warning(f"机器人状态初始化失败, 监控将停止.")
            return False

    async def on_trigger(self, message: Message, keys, reply):
        wr = async_partial(self.client.wait_reply, self.bot_username)
        for _ in range(3):
            msg = await wr(self.captcha)
            if "验证码错误" in msg.text:
                self.log.info(f"验证码错误, 将重新初始化.")
                if not await self.init():
                    return
            elif "暂时停止注册" in msg.text:
                self.log.info(f"注册名额已满, 将进行重试.")
                if not await self.init():
                    return
            elif "用户名" in msg.text:
                msg = await wr(self.unique_name)
                if "密码" in msg.text:
                    await self.client.send_message(self.bot_username, "/cancel")
                    self.log.bind(notify=True).info(f'已向Bot发送用户注册申请: "{self.unique_name}", 请检查结果.')
        else:
            self.log.info(f"未成功, 结束注册申请.")



import asyncio
import random
from pyrogram.types import Message
from thefuzz import process, fuzz

from ...data import get_data
from .base import AnswerBotCheckin


class JMSCheckin(AnswerBotCheckin):
    ocr = "idioms@v2"
    idioms = None
    lock = asyncio.Lock()

    name = "卷毛鼠"
    bot_username = "jmsembybot"

    async def start(self):
        self.retries = 2
        async with self.lock:
            if self.idioms is None:
                with open(
                    await get_data(self.basedir, "idioms@v1.txt", proxy=self.proxy, caller=self.name)
                ) as f:
                    self.__class__.idioms = [i for i in f.read().splitlines() if len(i) == 4]
        return await super().start()

    def to_idiom(self, captcha: str):
        phrase, score = process.extractOne(captcha, self.idioms, scorer=fuzz.partial_token_sort_ratio)
        if score > 70 or len(captcha) < 4:
            result = phrase
            self.log.debug(f'[gray50]已匹配识别验证码 "{captcha}" -> 成语 "{result}"[/]')
        else:
            result = captcha
        return result

    async def on_captcha(self, message: Message, captcha: str):
        captcha = self.to_idiom(captcha)
        async with self.operable:
            if not self.message:
                await self.operable.wait()
            await asyncio.sleep(random.randint(300, 500) / 100)
            for l in captcha:
                try:
                    await self.message.click(l)
                    await asyncio.sleep(random.randint(50, 300) / 100)
                except ValueError:
                    self.log.info(f'未能找到对应 "{l}" 的按键, 正在重试.')
                    await self.retry()
                    break



import asyncio
import random
import time
from typing import Callable, Coroutine, List, Optional, Tuple, Union
import uuid

import tomli
from loguru import logger
from pyrogram import filters
from pyrogram.handlers import MessageHandler
from pyrogram.types import Message

from ..utils import async_partial, truncate_str
from .tele import Client


class Link:
    bot = "embykeeper_auth_bot"

    def __init__(self, client: Client):
        self.client = client
        self.log = logger.bind(scheme="telelink", username=client.me.name)

    @property
    def instance(self):
        rd = random.Random()
        rd.seed(uuid.getnode())
        return uuid.UUID(int=rd.getrandbits(128))

    async def delete_messages(self, messages: List[Message]):
        async def delete(m: Message):
            try:
                await m.delete(revoke=True)
                text = m.text or m.caption or "图片或其他内容"
                text = truncate_str(text.replace("\n", ""), 30)
                self.log.debug(f"[gray50]删除了API消息记录: {text}[/]")
            except asyncio.CancelledError:
                pass

        return await asyncio.gather(*[delete(m) for m in messages])

    async def post(
        self, cmd, condition: Callable = None, timeout: int = 5, retries=3, name: str = None
    ) -> Tuple[Optional[str], Optional[str]]:
        for r in range(retries):
            self.log.debug(f"[gray50]禁用提醒 {timeout} 秒: {self.bot}[/]")
            await self.client.mute_chat(self.bot, time.time() + timeout + 5)
            future = asyncio.Future()
            handler = MessageHandler(
                async_partial(self._handler, cmd=cmd, future=future, condition=condition),
                filters.text & filters.bot & filters.user(self.bot),
            )
            await self.client.add_handler(handler, group=1)
            try:
                messages = []
                messages.append(await self.client.send_message(self.bot, f"/start quiet"))
                await asyncio.sleep(0.5)
                messages.append(await self.client.send_message(self.bot, cmd))
                self.log.debug(f"[gray50]-> {cmd}[/]")
                results = await asyncio.wait_for(future, timeout=timeout)
            except asyncio.CancelledError:
                try:
                    await asyncio.wait_for(self.delete_messages(messages), 1.0)
                except asyncio.TimeoutError:
                    pass
                finally:
                    raise
            except asyncio.TimeoutError:
                await self.delete_messages(messages)
                if r + 1 < retries:
                    self.log.info(f"{name}超时 ({r + 1}/{retries}), 将在 3 秒后重试.")
                    await asyncio.sleep(3)
                    continue
                else:
                    self.log.warning(f"{name}超时 ({r + 1}/{retries}).")
                    return None
            else:
                await self.delete_messages(messages)
                status, errmsg = [results.get(p, None) for p in ("status", "errmsg")]
                if status == "error":
                    self.log.warning(f"{name}错误: {errmsg}.")
                    return False
                elif status == "ok":
                    return results
                else:
                    self.log.warning(f"{name}出现未知错误.")
                    return False
            finally:
                await self.client.remove_handler(handler, group=1)

    async def _handler(
        self,
        client: Client,
        message: Message,
        cmd: str,
        future: asyncio.Future,
        condition: Union[bool, Callable[..., Coroutine], Callable] = None,
    ):
        try:
            toml = tomli.loads(message.text)
        except tomli.TOMLDecodeError:
            self.delete_messages([message])
        else:
            try:
                if toml.get("command", None) == cmd:
                    if condition is None:
                        cond = True
                    elif asyncio.iscoroutinefunction(condition):
                        cond = await condition(toml)
                    elif callable(condition):
                        cond = condition(toml)
                    if cond:
                        future.set_result(toml)
                        await asyncio.sleep(0.5)
                        await self.delete_messages([message])
                        return
            except asyncio.CancelledError as e:
                try:
                    await asyncio.wait_for(self.delete_messages([message]), 1)
                except asyncio.TimeoutError:
                    pass
                finally:
                    future.set_exception(e)
            finally:
                message.continue_propagation()

    async def auth(self, service: str):
        results = await self.post(f"/auth {service} {self.instance}", name=f"服务 {service.capitalize()} 认证")
        return bool(results)

    async def captcha(self):
        results = await self.post(f"/captcha {self.instance}", timeout=240, name="请求跳过验证码")
        if results:
            return [results.get(p, None) for p in ("token", "proxy", "useragent")]
        else:
            return None, None, None

    async def answer(self, question: str):
        results = await self.post(f"/answer {self.instance} {question}", timeout=10, name="请求问题回答")
        if results:
            return results.get("answer", None)

    async def send_log(self, message):
        results = await self.post(f"/log {self.instance} {message}", name="发送日志到 Telegram")
        return bool(results)
